#ifndef MPIPE_API_COMMON_H
#define MPIPE_API_COMMON_H

#include "mpipe_defs.h"
#include "mpipe_types.h"
#include <stddef.h>
#include <string.h>

EXTERN_C_BEGIN

// общий описатель ссылочных данных
typedef struct mpipe_data_t
{
    const void*                     data;
    size_t                          size;

} mpipe_data_t;

// координата точки на плоскости
typedef struct mpipe_position_t
{
    int32_t                         x;          // координата x относительно левого края плоскости
    int32_t                         y;          // координата y относительно верхнего края плоскости
} mpipe_position_t;

// размер плоскости
typedef struct mpipe_size_t
{
    int32_t                         width;      // ширина плоскости
    int32_t                         height;     // высота плоскости
} mpipe_size_t;

// область на плоскости
typedef struct mpipe_rect_t
{
    mpipe_position_t                offset;     // смещение координаты x и y относительно левого верхнего края плоскости
    mpipe_size_t                    size;       // ширина и высота области на плоскости
} mpipe_rect_t;

#define MPIPE_OBJECT_INIT(obj)      bzero(&obj, sizeof(obj))  // NOLINT
#define MPIPE_HANDLE_VALID(handle)  ((handle) >= 0)
#define MPIPE_RESULT_OK(result)     ((result) >= mpipe_result_ok)
#define MPIPE_MKTAG(a, b, c, d)   (a | (b << 8) | (c << 16) | (d << 24))

// удалить объект по номеру дескриптора
EXPORT_API enum mpipe_result_t mpipe_object_remove(mpipe_handle_t handle);

// возвращает тип объекта по номеру дескриптора
EXPORT_API enum mpipe_object_type_t mpipe_object_get_type(mpipe_handle_t handle);

EXTERN_C_END

#endif // MPIPE_API_COMMON_H
