#ifndef PORTABLE_SYNC_BASE_H
#define PORTABLE_SYNC_BASE_H

#if HAS_DL_DETECTOR
    #define __sl_detect_deadlock 1
#else
    #define __sl_detect_deadlock 0
#endif


#include <atomic>

namespace pt::utils
{

constexpr std::size_t default_max_idle_cycles = 1000;

class spin_lock
{
    std::size_t                 m_max_idle_cycles;
    std::atomic_flag            m_spin;
    std::atomic<std::size_t>    m_count;
#if __sl_detect_deadlock
    std::size_t                 m_thread_id;
#endif

public:
    spin_lock(std::size_t max_idle_cycles = default_max_idle_cycles);
    spin_lock(const spin_lock&) = delete;
    ~spin_lock();

    void lock();
    bool try_lock();
    void unlock();
};

class shared_spin_lock : public spin_lock
{
    std::atomic<std::size_t>    m_count;
public:
    shared_spin_lock(std::size_t max_idle_cycles = default_max_idle_cycles);
    shared_spin_lock(const spin_lock&) = delete;

    void lock();
    void lock_shared();
    bool try_lock_shared();
    void unlock_shared();
};

class fake_lock
{
    void lock();
    bool try_lock();
    void unlock();
};

}

#endif // PORTABLE_SYNC_BASE_H
