#ifndef MPIPE_UTILS_MEDIA_ROUTER_ASYNC_IMPL_H
#define MPIPE_UTILS_MEDIA_ROUTER_ASYNC_IMPL_H

#include "media_router_impl.h"
#include "core/task/i_task_manager.h"
#include "core/media/i_media_frame.h"
#include "core/command/i_command.h"



#include <queue>

namespace mpipe::utils::media
{

class media_router_async_impl : public media_router_impl
{
    struct storage_t
    {
        using queue_t = std::queue<storage_t>;
        i_media_frame::u_ptr_t  frame;
        i_command::u_ptr_t      command;
        timestamp_t             timestamp;

        storage_t(const i_media_frame& frame);
        storage_t(const i_command& command);
        storage_t();

        bool is_valid() const;
        operator bool () const;
    };

    mutable mutex_t                 m_queue_mutex;
    i_task_manager&                 m_task_manager;
    std::size_t                     m_max_queue_size;
    timestamp_t                     m_max_delay;
    std::size_t                     m_drop;
    storage_t::queue_t              m_queue;
    bool                            m_sync_mode;

    std::atomic_bool                m_processed;
public:
    media_router_async_impl(i_task_manager& task_manager
                              , std::size_t max_queue_size
                              , timestamp_t max_delay = timestamp_null
                              , bool sync_mode = false);
    ~media_router_async_impl();


    bool is_processed() const;
    bool is_sync_mode() const;
    void set_sync_mode(bool sync_mode);
    std::size_t drop_messages() const;
    std::size_t pending_messages() const;

    // i_message_sink interface
public:
    bool send_frame(const i_media_frame &frame) override;
    bool send_command(const i_command &command) override;
protected:
    virtual bool on_send(storage_t&& storage);
private:
    void check_or_start();
    void on_async_proc();
    bool enqueue(storage_t&& storage);
    storage_t::queue_t fetch_queue();
    std::size_t check_and_drop_queue();
};

}

#endif // MPIPE_UTILS_MEDIA_ROUTER_ASYNC_IMPL_H
