#ifndef MPIPE_UTILS_OPTION_WRITER_H
#define MPIPE_UTILS_OPTION_WRITER_H

#include "option_reader.h"

namespace mpipe::utils::option
{

class option_writer : public option_reader
{
public:
    option_writer(i_option& root);

    i_option* get_child(const std::string& key);

    bool set(const std::string& key, i_option::u_ptr_t&& option);
    bool remove(const std::string& key);

    template<typename T>
    inline bool set(const std::string& key, const T& value)
    {
        //return set(key, serialize(value));
        if (auto v = serialize(value))
        {
            return set(key, std::move(v));
        }

        set(key, i_option::u_ptr_t(nullptr));

        return true;
    }

    template<typename T>
    inline bool set(const std::string& key, const T& value, const T& default_value)
    {
        if (value != default_value)
        {
            return set(key, value);
        }

        return true;
    }

    template<typename T, typename ...Args>
    inline bool set_variant(const std::string& key, const std::variant<Args...>& value)
    {
        if (auto v = std::get_if<T>(value))
        {
            return set(key, *v);
        }

        return false;
    }


private:
    i_option& root();
};

}

#endif // MPIPE_UTILS_OPTION_WRITER_H
