#ifndef NET_LISTENER_IMPL_H
#define NET_LISTENER_IMPL_H

#include "net/i_transport.h"
#include <functional>

namespace mpipe::net::utils
{

class transport_listener_impl : public i_transport::i_listener
{
    using packet_handler_t = std::function<void(const i_net_packet&)>;
    using state_handler_t = std::function<void(channel_state_t, const std::string_view&)>;
    using transport_handler_t = std::function<void(i_transport::u_ptr_t &&transport)>;

    packet_handler_t        m_packet_handler;
    state_handler_t         m_state_handler;
    transport_handler_t     m_transport_handler;

    // i_listener interface
public:
    transport_listener_impl(packet_handler_t&& packet_handler = nullptr
                            , state_handler_t&& state_handler = nullptr
                            , transport_handler_t&& transport_handler = nullptr);

    void set_packet_handler(packet_handler_t&& packet_handler);
    void set_state_handler(state_handler_t&& state_handler);
    void set_transport_handler(transport_handler_t&& transport_handler);

    void on_packet(const i_net_packet &packet) override;
    void on_state(channel_state_t state, const std::string_view &reason) override;
    void on_transport(i_transport::u_ptr_t&& transport) override;
};

}


#endif // NET_LISTENER_IMPL_H
