#ifndef MPIPE_UTILS_QUEUE_IMPL_H
#define MPIPE_UTILS_QUEUE_IMPL_H

#include "core/i_queue.h"
#include "core/time/time_types.h"

#include <queue>

namespace mpipe::utils
{

struct queue_config_t
{
    std::size_t                         max_queue_size;
    timestamp_t                         max_delay;

    queue_config_t(std::size_t max_queue_size = 0
                  , timestamp_t max_delay = timestamp_null);
};

template<class T, class Mutex>
class queue_impl : public i_queue<T>
{
    using value_type_t = typename i_queue<T>::value_type_t;
    struct value_storage_t
    {
        using queue_t = std::queue<value_storage_t>;
        timestamp_t                     timestamp;
        value_type_t                    value;

        value_storage_t(value_type_t&& value);
    };

    using mutex_t = Mutex;

    mutable mutex_t                     m_safe_mutex;
    queue_config_t                      m_config;
    typename value_storage_t::queue_t   m_queue;
    std::size_t                         m_drop;
public:
    queue_impl(const queue_config_t& config = {});
    const queue_config_t& config() const;
    // i_queue interface
public:
    bool push(value_type_t &&value) override;
    value_type_t pop() override;
    std::size_t pending() const override;
    std::size_t drop() const override;
    std::size_t capacity() const override;
    void reset() override;
private:
    std::size_t drop_by_delay();
    std::size_t drop_by_size();

};

}

#endif // MPIPE_UTILS_QUEUE_IMPL_H
